"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const t = require("@babel/types");
const generator_1 = require("@babel/generator");
const traverse_1 = require("@babel/traverse");
const scriptToAST_1 = require("../scriptToAST");
function isFunctionObjectProperty(node) {
    return t.isFunctionExpression(node.value) || t.isArrowFunctionExpression(node.value);
}
function getUsed(ast) {
    const programedFunc = scriptToAST_1.defaultParse(`var a = {${generator_1.default(ast).code}}`);
    const used = [];
    traverse_1.default(programedFunc, {
        ThisExpression(path) {
            used.push(generator_1.default(path.parent).code);
        }
    });
    return used;
}
function processFunctionProperty(node) {
    if (t.isObjectProperty(node) && isFunctionObjectProperty(node)) {
        return {
            name: node.key.name,
            code: generator_1.default(node.value).code,
            use: getUsed(node)
        };
    }
    else if (t.isObjectMethod(node)) {
        return {
            name: node.key.name,
            code: generator_1.default(node).code,
            use: getUsed(node)
        };
    }
    console.warn('node is not a function');
    return {
        name: '',
        code: '',
        use: []
    };
}
exports.default = processFunctionProperty;
function isFunctionProperty(node) {
    if (t.isObjectProperty(node)) {
        if (t.isFunctionExpression(node.value) || t.isArrowFunctionExpression(node.value)) {
            return true;
        }
    }
    else if (t.isObjectMethod(node)) {
        return true;
    }
    return false;
}
exports.isFunctionProperty = isFunctionProperty;
//# sourceMappingURL=processFunctionProperty.js.map