"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const traverse_1 = require("@babel/traverse");
const t = require("@babel/types");
const scriptToAST_1 = require("./scriptToAST");
const getConcatedComments_1 = require("./helpers/getConcatedComments");
const processFunctionProperty_1 = require("./helpers/processFunctionProperty");
const ComponentTypes_1 = require("./ComponentTypes");
const nameReader_1 = require("./readers/nameReader");
const propsReader_1 = require("./readers/propsReader");
const dataReader_1 = require("./readers/dataReader");
const methodsReader_1 = require("./readers/methodsReader");
const computedReader_1 = require("./readers/computedReader");
const watchReader_1 = require("./readers/watchReader");
const lifeCyclesReader_1 = require("./readers/lifeCyclesReader");
function default_1(code) {
    const ast = scriptToAST_1.default(code);
    const mainCommentArr = ast.comments.reduce((sofar, comment, index, arr) => {
        if (comment.type === 'CommentBlock' && arr.indexOf(sofar[sofar.length - 1]) + 1 === index) {
            return [...sofar, comment];
        }
        else {
            return sofar;
        }
    }, []);
    const comment = getConcatedComments_1.default(mainCommentArr);
    const dependencies = [];
    let name;
    let props = [];
    let data = [];
    let methods = [];
    let computed = [];
    let watch = [];
    let lifeCycles = [];
    traverse_1.default(ast, {
        ImportDeclaration(path) {
            const source = path.node.source.value;
            const specifiers = path.node.specifiers.map(s => {
                return {
                    type: s.type,
                    name: s.local.name
                };
            });
            const comment = getConcatedComments_1.default(path.node.leadingComments ? path.node.leadingComments.filter(comment => {
                return !mainCommentArr.includes(comment);
            }) : []);
            const dep = {
                source,
                specifiers,
                comment
            };
            dependencies.push(dep);
        },
        ExportDefaultDeclaration(rootPath) {
            const declaration = rootPath.node.declaration;
            const properties = declaration.properties;
            function isVueOptionNameSetAsMethod(opName) {
                return ['data', ...ComponentTypes_1.lifeCycleName].includes(opName);
            }
            function isVueOptionNameSetAsProperty(opName) {
                return ['name', 'props', 'computed', 'watch', 'methods'].includes(opName);
            }
            const nodeOfVueOptions = new Map();
            properties.forEach(pNode => {
                if (processFunctionProperty_1.isFunctionProperty(pNode) && isVueOptionNameSetAsMethod(pNode.key.name)) {
                    nodeOfVueOptions.set(pNode.key.name, pNode);
                }
                else if (t.isObjectProperty(pNode) && isVueOptionNameSetAsProperty(pNode.key.name)) {
                    nodeOfVueOptions.set(pNode.key.name, pNode);
                }
            });
            name = nameReader_1.default(nodeOfVueOptions);
            props = propsReader_1.default(nodeOfVueOptions);
            data = dataReader_1.default(nodeOfVueOptions);
            methods = methodsReader_1.default(nodeOfVueOptions);
            computed = computedReader_1.default(nodeOfVueOptions);
            watch = watchReader_1.default(nodeOfVueOptions);
            lifeCycles = lifeCyclesReader_1.default(nodeOfVueOptions);
        }
    });
    const ci = {
        comment,
        name,
        dependencies,
        props,
        data,
        computed,
        methods,
        watch,
        lifeCycles
    };
    return ci;
}
exports.default = default_1;
//# sourceMappingURL=index.js.map