"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const t = require("@babel/types");
const generator_1 = require("@babel/generator");
const getConcatedComments_1 = require("../helpers/getConcatedComments");
const processFunctionProperty_1 = require("../helpers/processFunctionProperty");
function default_1(nodeOfVueOptions) {
    const node = nodeOfVueOptions.get('props');
    if (node) {
        if (t.isArrayExpression(node.value)) {
            const elements = node.value.elements;
            return elements.map(propNode => {
                return {
                    name: propNode.value
                };
            });
        }
        if (t.isObjectExpression(node.value)) {
            const properties = node.value.properties;
            return properties.map(propNode => {
                if (t.isObjectProperty(propNode)) {
                    if (t.isIdentifier(propNode.value) || t.isArrayExpression(propNode.value)) {
                        const prop = {
                            name: propNode.key.name,
                            type: generator_1.default(propNode.value).code,
                            comment: getConcatedComments_1.default(propNode.leadingComments || [])
                        };
                        return prop;
                    }
                    else if (t.isObjectExpression(propNode.value)) {
                        let prop = {
                            name: propNode.key.name,
                            comment: getConcatedComments_1.default(propNode.leadingComments || [])
                        };
                        propNode.value.properties.forEach(propOptionNode => {
                            if (t.isSpreadElement(propOptionNode)) {
                                return;
                            }
                            const propOptionName = propOptionNode.key.name;
                            if (t.isObjectProperty(propOptionNode) && propOptionName === 'type') {
                                prop.type = generator_1.default(propOptionNode.value).code;
                            }
                            if (t.isObjectProperty(propOptionNode) && propOptionName === 'required') {
                                prop.required = propOptionNode.value.value;
                            }
                            if (t.isObjectProperty(propOptionNode) && propOptionName === 'default') {
                                if (t.isNumericLiteral(propOptionNode.value) || t.isStringLiteral(propOptionNode.value) || t.isBooleanLiteral(propOptionNode.value)) {
                                    prop.default = propOptionNode.value.value;
                                }
                                else if (t.isNullLiteral(propOptionNode.value)) {
                                    prop.default = null;
                                }
                                else if (t.isIdentifier(propOptionNode.value) && propOptionNode.value.name === 'undefined') {
                                    prop.default = undefined;
                                }
                                else if (t.isFunctionExpression(propOptionNode.value) || t.isArrowFunctionExpression(propOptionNode.value)) {
                                    prop.default = processFunctionProperty_1.default(propOptionNode);
                                }
                                else {
                                    console.warn('default not set as any of number,string,boolean,null,undefined,generator function');
                                }
                            }
                            if (t.isObjectMethod(propOptionNode) && propOptionName === 'default') {
                                prop.default = processFunctionProperty_1.default(propOptionNode);
                            }
                            if ((t.isObjectProperty(propOptionNode) || t.isObjectMethod(propOptionNode)) && propOptionName === 'validator') {
                                prop.validator = processFunctionProperty_1.default(propOptionNode);
                            }
                        });
                        return prop;
                    }
                    else {
                        console.warn('some props value is invalid');
                        return null;
                    }
                }
                else {
                    console.warn('some props do not set as object property');
                    return null;
                }
            }).filter((v) => !!v);
        }
    }
    return [];
}
exports.default = default_1;
//# sourceMappingURL=propsReader.js.map